#include <cmath>

using std::log;
using std::exp;


double log_2(double x)
{
	return log(x) / 0.69314718055994529;
	//return log(x) / log(2.0);
	
}

// natural logarithm of 2-tailed p-value with respect to binomial distribution
// n: nr of samples; p: probability of success; s: nr of observed successes
// p-value = Pr( B(n,p;x) <= B(n,p;s) )
double log_pvalue(int n, double p, int s)
{
	double log_pvalue, log_tail2, log_prob;
	
	const double log_yes = log(p);
	const double log_no  = log(1.0 - p);
	
	if (int(p*n) == s) {
		return 0.0;
	}
	else if (s > p * n) {
		double log_term = n * log_yes;
		log_pvalue = log_term;
		for (int i=n-1; i>=s; --i) {
			log_term += log(i+1) - log(n-i);
			log_term += - log_yes + log_no;
			log_pvalue = log(exp(log_pvalue-log_term) + 1.0) + log_term;
		}
		log_prob = log_term;
		log_term = n * log_no;
		int i = 1;
		log_tail2 = log(0.0);
		while (log_term < log_prob) {
			log_tail2 = log(exp(log_tail2-log_term) + 1.0) + log_term;
			log_term += log(n-i+1) - log(i);
			log_term += - log_no + log_yes;
			++i;
		}
		log_pvalue = log(exp(log_tail2-log_pvalue) + 1.0) + log_pvalue;
	}
	else {
		double log_term = n * log_no;
		log_pvalue = log_term;
		for (int i=1; i<=s; ++i) {
			log_term = log_term + log(n-i+1) - log(i);
			log_term = log_term - log_no + log_yes;
			log_pvalue = log(exp(log_pvalue-log_term) + 1.0) + log_term;
		}
		log_prob = log_term;
		log_term = n * log_yes;
		int i = n - 1;
		log_tail2 = log(0.0);
		while (log_term < log_prob) {
			log_tail2 = log(exp(log_tail2-log_term) + 1.0) + log_term;
			log_term = log_term + log(i+1) - log(n-i);
			log_term = log_term - log_yes + log_no;
			--i;
		}
		log_pvalue = log(exp(log_tail2-log_pvalue) + 1.0) + log_pvalue;
	}
	
	return log_pvalue;
}
