#include <mpfr.h>
#include <iostream>
#include <math.h>

using namespace std;

class Binomial
{
public:
	Binomial(int n, double p) : n(n), p(p)
	{
		mpfr_init2(prob, 256);
		mpfr_init2(cumulative, 256);
	};
	
	
	~Binomial()
	{
		mpfr_clear(prob);
		mpfr_clear(cumulative);
	};
	
	
	double pvalue(double f)
	{
		// two-tailed p-value
		if (f == p) {
			return 1.0;
		}
		else if (f < p) {
			int k = (int)floor(f*n);
			double prk = probability(k), prk2;
			int lo=(int)ceil(p*n), hi=n, k2=(lo+hi)>>1;
			while (lo < hi) {
				prk2 = probability(k2);
				if (prk2 < prk)
					hi = k2-1;
				else if (prk2 > prk)
					lo = k2+1;
				else
					break;
				k2 = (lo+hi)>>1;
			}
			if (k2-k <= 1)
				return 1.0;
			double pv = cumulative_down(k) + cumulative_up(k2);
			if (pv > 1) { // infinity due to stuff but should be one ...
				pv = 1.0;
			}
			return pv;
		}
		else {
			int k = (int)ceil(f*n);
			double prk = probability(k), prk2;
			int lo=0, hi=(int)floor(p*n), k2=(lo+hi)>>1;
			while (lo < hi) {
				prk2 = probability(k2);
				if (prk2 < prk)
					lo = k2+1;
				else if (prk2 > prk)
					hi = k2-1;
				else
					break;
				k2 = (lo+hi)>>1;
			}
			if (k-k2 <= 1)
				return 1.0;
			double pv = cumulative_up(k) + cumulative_down(k2);
			if (pv > 1) { // infinity due to stuff but should be one ...
				pv = 1.0; 
			}
			return pv;
		}
		
	};
	
	
	double probability(int k)
	{
		mpfr_set_d(prob, 1.0 - p, GMP_RNDN);
		mpfr_pow_ui(prob, prob, n, GMP_RNDN);
		for (int i=1; i<=k; ++i)
			mpfr_mul_d(prob, prob, p/(1-p) * (n-i+1)/i, GMP_RNDN);
		
		return mpfr_get_d(prob, GMP_RNDN);
	}
	
	
	double cumulative_down(int k)
	{
		if (k < 0) return 0.0;
		mpfr_set_d(prob, 1.0 - p, GMP_RNDN);
		mpfr_pow_ui(prob, prob, n, GMP_RNDN);
		mpfr_set(cumulative, prob, GMP_RNDN);
		
		for (int i=1; i<=k; ++i) {
			mpfr_mul_d(prob, prob, p/(1.0-p) * (n-i+1)/i, GMP_RNDN);
			mpfr_add(cumulative, cumulative, prob, GMP_RNDN);
		}
		
		return mpfr_get_d(cumulative, GMP_RNDN);
	}
	
	
	double cumulative_up(int k)
	{
		if (k > n) return 0.0;
		mpfr_set_d(prob, p, GMP_RNDN);
		mpfr_pow_ui(prob, prob, n, GMP_RNDN);
		mpfr_set(cumulative, prob, GMP_RNDN);
		
		for (int i=1; i<=n-k; ++i) {
			mpfr_mul_d(prob, prob, (1.0-p)/p * (n-i+1)/i, GMP_RNDN);
			mpfr_add(cumulative, cumulative, prob, GMP_RNDN);
		}
		
		return mpfr_get_d(cumulative, GMP_RNDN);
	}
	
	
	int n;
	double p;
	
	mpfr_t prob;
	mpfr_t cumulative;
	
};
