#ifndef _BITVECTOR_H_
#define _BITVECTOR_H_


#include <string.h>
#include <iostream>

using namespace std;

class BitVector
{
public:
	BitVector() : bit_cnt(0) {
		bits = new unsigned int[s];
		memset(bits, 0, s * bytesperword);
	}
	
	~BitVector() {
		delete [] bits;
	}
	
	BitVector(const BitVector & other) {
		bits = new unsigned int[s];
		copy_bits(other);
		return;
	}
	
	BitVector & operator=(const BitVector & other) {
		copy_bits(other);
		return *this;
	}
	
	void copy_bits(const BitVector & other) {
		memcpy(bits, other.bits, s * bytesperword);
		bit_cnt = other.bit_cnt;
	}
	
	void set_bit(unsigned int bit) {
		bits[bit/bitsperword] |= (1 << (bit % bitsperword));
	}
	
	void count_bits() {
		unsigned int v;
		bit_cnt = 0;
		for (int i=0; i<s; ++i) {
			v = bits[i];
			for ( ; v; ++bit_cnt)
				v &= v - 1;
		}
	}
	
	bool isset(unsigned int bit) const {
		return bits[bit/bitsperword] & (1 << (bit % bitsperword));
	}
	
	bool contains(const BitVector & other) const {
		for (int i=0; i<s; ++i) {
			if ((bits[i] & other.bits[i]) != other.bits[i])
				return false;
		}
		return true;
	}
	
	void update(const BitVector & other) {
		for (int i=0; i<s; ++i) {
			bits[i] |= other.bits[i];
		}
		return;
	}
	
	int max_bit() const {
		int i = s - 1;
		while (i>=0 && bits[i]) --i; // bits[i] != 0 ?
		if (i < 0) return -1;
		unsigned int c = bitsperword - 1;
		while (c >= 0 && !isset(c)) --c;
		
		return i * bitsperword + c;
	}
	
	void clear() {
		for (int i=0; i<s; ++i)
			bits[i] = 0;
		
		return;
	}
	
	static void set_size(int new_size) {
		size = new_size;
		s = (size+bitsperword-1) / bitsperword; // ceil(size/bitsperword)
	}
	
	
	
public:
	unsigned int *bits;
	int bit_cnt;
	
	static int size;
	static int s;
	static const int bytesperword = sizeof(unsigned int);
	static const int bitsperword  = 8 * bytesperword;
	
};


#endif //_BITVECTOR_H_
