#ifndef _EXPMODELGRAPH_H_
#define _EXPMODELGRAPH_H_

#include "expmodel.h"

class ExpModelGraph
{
public:
	ExpModelGraph(uint n, uint d, bool use_items = false, bool use_rowmargins = false);
	~ExpModelGraph();
	
	void add_itemset(int *itemset, int len, double freq);
	void set_item_frequency(int item, double frequency);
	void set_rowsize_prob(uint size, double prob);
	
	double estimate_frequency(int *itemset, int len);
	
	void iterative_scaling(int maxloops=100);
	
	void delete_itemset(int j);
	
	double get_entropy();
	double bg_entropy();
	
	void print_graph();
	
	
public:
	void compute_rowsize_probs();
	void compute_background_params();
	
public:
	ExpModel **nodes;
	uint node_count;
	
	uint item_cnt;
	uint trans_cnt;
	bool include_items;
	bool include_rowmargins;
	
	Itemset **all_itemsets;
	double *frequencies;
	uint all_itemsets_size;
	
	double *item_frequencies;
	double *rowsize_probs;
	
	uint *item_nodes;
	
	mpfr_t estimate;
	mpfr_t f;
	mpfr_t tmp;
	mpfr_t h;
	
	int  *projectedset;
	uint *thegroups;
	uint groupsize;
	
	static const unsigned int max_k = 200;
	
};

#endif // _EXPMODELGRAPH_H_
