#include "partitionnode.h"
#include <iostream>
#include <string.h>
#include <math.h>

using namespace std;

int  PartitionNode::n = 1;
int  PartitionNode::d = 1;
bool PartitionNode::use_items      = false;
bool PartitionNode::use_rowmargins = false;
int  PartitionNode::size_prec = 1;
int  PartitionNode::prob_prec = 4;


PartitionNode::PartitionNode()
{
	mpfr_init2(size, size_prec);
	mpfr_init2(prob, prob_prec);
	if (use_items || use_rowmargins)
		mpfr_init2(cum_size, prob_prec);
	
	if (use_rowmargins) {
		size_array     = new mpfr_t[n+1];
		cum_size_array = new mpfr_t[n+1];
		for (int i=0; i<=n; ++i) {
			mpfr_init2(cum_size_array[i], prob_prec);
			mpfr_init2(size_array[i], prob_prec);
		}
	}
	
	return;
}


PartitionNode::~PartitionNode()
{
	mpfr_clear(size);
	mpfr_clear(prob);
	if (use_items || use_rowmargins) {
		mpfr_clear(cum_size);
	}
	
	if (use_rowmargins) {
		for (int i=0; i<=n; ++i) {
			mpfr_clear(cum_size_array[i]);
			mpfr_clear(size_array[i]);
		}
		delete [] cum_size_array;
		delete [] size_array;
	}
	
	return;
}


PartitionNode & PartitionNode::operator=(const PartitionNode & other)
{
	mpfr_set(size, other.size, GMP_RNDN);
	mpfr_set(prob, other.prob, GMP_RNDN);
	if (use_items || use_rowmargins)
		mpfr_set(cum_size, other.cum_size, GMP_RNDN);
	
	fixed = other.fixed;
	
	if (use_rowmargins) {
		for (int i=0; i<=n; ++i) {
			mpfr_set(cum_size_array[i], other.cum_size_array[i], GMP_RNDN);
			mpfr_set(size_array[i], other.size_array[i], GMP_RNDN);
		}
	}
	
	return *this;
}


void PartitionNode::set_n(int new_n)
{
	n = new_n;
	BitVector::set_size(n);
	
	return;
}

void PartitionNode::set_d(int new_d)
{
	d = new_d;
	
	return;
}


void PartitionNode::set_mode(bool use_i, bool use_r)
{
	use_items      = use_i;
	use_rowmargins = use_r;
	
	if (use_items || use_rowmargins) {
		double ld = log(d+1)/log(2) + 1;
		size_prec = min(n, 64) * ld;
		prob_prec = min(n, 64) * ld;
	}
	else {
		size_prec = n * 1;
		prob_prec = n * 4;
	}
	
	return;
}
