#include <string>
#include "expmodelgraph.h"
#include <set>
#include <map>
#include "tidlist.h"
//#include "trie.h"
#include <cmath>
#include "auxiliary.h"

using namespace std;



struct ItemFrequency
{
	int item;
	double frequency;
};


class SummaryMiner
{
public:
	SummaryMiner(int verbosity=1, int topk=0);
	~SummaryMiner();
	
	void read_data(string filename);
	int seed_summary(string filename);
	
	void mine_and_store_frequent_sets(TidList **items, int tailsize, int depth=0);
	
	void mine_summary(int k, bool lowstop=false, int timelimit=0, bool onthefly=true);
	
	uint mine_best_itemset(TidList **items, int tailsize, int clen = 0, int depth=0);
	uint mine_best_itemset_offthefly(ItemFrequency *items, int tailsize, int clen=0, int depth=0);
	
	void read_dictionary(string dictionaryfilename = string(""));
	void print_itemset(int i) const;
	void print_summary() const;
	void save_itemset(int i, FILE *outfile) const;
	void save_summary(string outfilename) const;
	
	void estimate_itemsets(string est_in_filename, string est_out_filename);
	
	void set_minsup(double thresh)   { minsup    = thresh; absminsup = minsup * trans_count; };
	void set_maxsize(int thresh)     { maxsize   = thresh; };
	void set_use_items(bool use_em)  { use_items = use_em; };
	void set_maxtreewidth(int mtw)   { maxtreewidth = mtw; };
	void set_maxgroupsize(int mgs)   { maxgroupsize = mgs; };
	void set_use_rowmargins(bool use_rm) { use_rowmargins = use_rm; };
	
	void set_penalty(int p) { penalty_type = p; };
	
	double get_penalty(int k=-1, int x=-1) const;
	double get_bic_penalty(int k=-1) const;
	double get_mdl_penalty_1(int k=-1) const;
	double get_mdl_penalty_2(int k=-1, int x=-1) const;
	double get_mdl_penalty_2_delta() const { return log_2(item_count); };
	
	static double heuristic(double freq1, double freq2)
	{
		if (freq1==freq2)
			return 0.0;
		if (freq1==0.0)
			return -log_2(1.0-freq2);
		if (freq1==1.0)
			return -log_2(freq2);
		return freq1 * log_2(freq1/freq2) + (1.0-freq1) * log_2((1.0-freq1)/(1.0-freq2));
	};
	
	double data_entropy();
	
	void insert_top_k_itemset(Itemset & iset, double score, double freq, double est);
	void print_top_k() const;
	
	
public:
	void init_summary();
	
	Itemset top_itemset;
	double top_score;
	double top_freq;
	double top_est;
	
	
	bool do_top_k_sets;
	int top_k;
	Itemset *top_k_itemsets;
	double *top_k_scores;
	double *top_k_freqs;
	double *top_k_ests;
	
	string datafilename;
	uint item_count;
	double trans_count;
	int freq_item_count;
	
	map<int,int> item2id;
	int *id2item;
	
	double minsup;
	double absminsup;
	int maxsize;
	bool use_items;
	bool use_rowmargins;
	uint maxtreewidth;
	uint maxgroupsize;
	
	int penalty_type;
	
	int *prefix;
	int *closure;
	int *prefix_nodes;
	int *closure_nodes;
	
	double bg_entropy;
	
	ExpModelGraph *summary;
	TidList **tidlists;
	
	double *rowsize_probs;
	
	Trie<double> *freq_trie;
	
	string *dict;
	
	int verbosity;
};
