#include <vector>
#include <algorithm>
#include <string.h>

using std::vector;
using std::sort;

class TidList
{
public:
	TidList(int id, const vector<int> & list) : id(id), size(list.size()), len(size) {
		tids = new int[len];
		for (int i=0; i<size; ++i) {
			tids[i] = list[i];
		}
	};
	TidList(int id, const TidList & one, const TidList & two) : id(id), size(0), len(0) {
		// init w difference
		tids = new int[one.len];
		int *i1f = one.tids, *i2f = two.tids;
		const int *i1l = i1f+one.len;
		const int *i2l = i2f+two.len;
		while (i1f < i1l && i2f < i2l) {
			if (*i1f < *i2f)
				tids[len++] = *i1f++;
			else if (*i2f < *i1f)
				++i2f;
			else {
				++i1f;
				++i2f;
			}
		}
		while (i1f < i1l)
			tids[len++] = *i1f++;
	}
	~TidList() {
		delete [] tids;
	};
	
	int id;
	int size; // support (!= len!)
	
	int *tids;
	int len; // length of array
	
	
	static int multi_intersect_size(TidList *head, TidList **tail, int tailsize) {
		int tmplen = head->len;
		memcpy(tmptids, head->tids, tmplen * sizeof(int));
		for (int j=0; j<tailsize; ++j) {
			int len2 = tail[j]->len;
			int t1 = 0, t2 = 0;
			int gap = 0;
			while (t1<tmplen && t2<len2) {
				if (tmptids[t1] == tail[j]->tids[t2]) {
					tmptids[t1-gap] = tmptids[t1];
					++t1;
					++t2;
				}
				else if (tmptids[t1] < tail[j]->tids[t2]) {
					++gap;
					++t1;
				}
				else {
					++t2;
				}
			}
			tmplen -= gap;
		}
		return tmplen;
	}
	
	static int multi_union_diff_size(TidList *head, TidList **tail, int tailsize) {
		if (tailsize == 0)
			return 0;
		int tmplen = tail[0]->len;
		memcpy(tmptids, tail[0]->tids, tmplen * sizeof(int));
		for (int j=1; j<tailsize; ++j) {
			int len2 = tail[j]->len;
			int t1 = 0, t2 = 0;
			int add = 0;
			while (t1<tmplen && t2<len2) {
				if (tmptids[t1] == tail[j]->tids[t2]) {
					++t1;
					++t2;
				}
				else if (tmptids[t1] < tail[j]->tids[t2]) {
					++t1;
				}
				else {
					tmptids[tmplen+add] = tail[j]->tids[t2];
					++add;
					++t2;
				}
			}
			tmplen += add;
			sort(tmptids, tmptids+tmplen);
		}
		int t1 = 0, t2 = 0;
		int gap = 0;
		int headlen = head->len;
		while (t1 < tmplen && t2 < headlen) {
			if (tmptids[t1] == head->tids[t2]) {
				++gap;
				++t1;
				++t2;
			}
			else if (tmptids[t1] < head->tids[t2]) {
				//tmptids[t1-gap] = tmptids[t1];
				++t1;
			}
			else {
				++t2;
			}
		}
		return tmplen - gap;
	}
	
	static void prepare(int trans_cnt);
	static void unprepare();
	
	static int *tmptids;
};

